@@ -1,6 +1,6 @@
 from typing import Any, Dict
 
-from pydantic import ValidationError
+from pydantic import ValidationError, Extra
 from ruamel.yaml.error import MarkedYAMLError
 import sqlalchemy
 
@@ -10,6 +10,9 @@ class Error:
     code: str
     details: Dict = dict()
 
+    class Config:
+        extra = Extra.forbid
+
     def __init__(self, kind, code, details):
         self.kind = kind
         self.code = code
@@ -24,6 +27,9 @@ class Result:
     value: Any = None
     error: Error = None
 
+    class Config:
+        extra = Extra.forbid
+
     def __init__(self, value: Any = None, error: Error = None):
         if error is not None:
             self.is_ok = False
